<?php
/**
 * @version		$Id: similarity_base.class.php,v 1.19 2010/06/18 20:11:08 juanca Exp $
 * @package		vpl. Similarity base and utility clases
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino
 **/

require_once dirname(__FILE__).'/tokenizer_factory.class.php';
require_once dirname(__FILE__).'/../views/status_box.class.php';

/**
 * @abstract class to represent files from any source
 */
class vpl_file_from_base{
	public function show_info(){
	}
	public function can_access(){
		return false;
	}
	public function get_userid(){
		return '';
	}
}

/**
 * Information of a file from a directory
 */
class vpl_file_from_dir extends vpl_file_from_base{
	static $usersname=array();
	var $dirname;
	var $filename;
	var $userid;
	//This is for compatibility with GAP 2.x application
	static public function process_gap_userfile($filepath){
		if(strtolower(basename($filepath)) == 'datospersonales.gap'){
			$NIF = '';
			$nombre='';
			$apellidos='';
			$lines = explode("\n",file_get_contents($filepath));
			if(count($lines)>3){
				if(strpos($lines[1],'NIF=')!==false){
					$NIF = substr($lines[1],4);
				}
				if(strpos($lines[2],'Nombre=')!==false){
					$nombre = substr($lines[2],7);
				}
				if(strpos($lines[3],'Apellidos=')!==false){
					$apellidos = substr($lines[3],10);
				}
			}
			if($NIF>'' && $nombre>'' && $apellidos>''){
				global $CFG;
				if ($CFG->fullnamedisplay == 'lastname firstname') {
					self::$usersname[$NIF]=mb_convert_encoding($apellidos.', '.$nombre,'utf-8');
				} else {
					self::$usersname[$NIF]=mb_convert_encoding($nombre.' '.$apellidos,'utf-8');
				}
			}
		}
	}

	//This is for compatibility with GAP 2.x application
	static public function get_user_id_from_file($filename){
		if(count(self::$usersname)){
			$filename = strtolower($filename);
			foreach(self::$usersname as $userid => $userdata){
				if(strpos($filename,$userid) !== false){
					return $userid;
				}
			}
		}
		return '';
	}

	function __construct(&$filename,$dirname,$userid=''){
		$this->filename=$filename;
		$this->dirname=$dirname;
		$this->userid=self::get_user_id_from_file($filename);
	}

	function get_userid(){
		return $this->userid;
	}
	public function show_info(){
		$ret='';
		$ret .='<a href="'.'file.php'.'">';
		$ret .=s($this->filename).' ';
		$ret .= '</a>';
		if($this->userid!=''){
			$ret .= ' '.self::$usersname[$this->userid];
		}
		return $ret;
	}
	public function can_access(){
		return true;
	}
	public function link_parms($t){
		$res = array('type'.$t=>2,'dirname'.$t => $this->dirname,'filename'.$t => $this->filename);
		if($this->userid!=''){
			$res['username'.$t]=self::$usersname[$this->userid];;
		}
		return $res;
	}
}

/**
 * Information of a file from a zip file
 *
 */
class vpl_file_from_zipfile extends vpl_file_from_dir{
	function __construct(&$filename,$zipname,$userid=''){
		parent::__construct($filename,$zipname,$userid);
	}
	public function show_info(){
		$ret='';
		$ret .=s($this->filename);
		if($this->userid!=''){
			$ret .= ' '.self::$usersname[$this->userid];
		}
		return $ret;
	}
	public function can_access(){
		return true;
	}
	public function link_parms($t){
		$res = array('type'.$t=>3,'zipfile'.$t=>$this->dirname,'filename'.$t => $this->filename);
		if($this->userid!=''){
			$res['username'.$t]=self::$usersname[$this->userid];;
		}
		return $res;
	}
}

/**
 * Information of a file from other vpl activity
 *
 */
class vpl_file_from_activity extends vpl_file_from_base{
	static $vpls=array();
	var $vplid;
	var $filename;
	var $subid;
	var $userid;
	function __construct(&$filename,&$vpl,$submistance){
		$id = $vpl->get_instance()->id;
		if(!isset(self::$vpls[$id])){
			self::$vpls[$id]=$vpl;
		}
		$this->vplid = $id;
		$this->filename=$filename;
		$this->userid=$submistance->userid;
		$this->subid=$submistance->id;
	}
	public function show_info(){
		$vpl = self::$vpls[$this->vplid];
		$cmid = $vpl->get_course_module()->id;
		$ret='';
		if($this->userid>=0){
			$user = get_record('user','id',$this->userid);
		}else{
			$user = false;
		}
		if($user){
			$ret .='<a href="'.vpl_mod_href('/forms/submissionview.php','id',$cmid,'userid',$user->id).'">';
		}
		$ret .=s($this->filename).' ';
		if($user){
			$ret .= '</a>';
			$ret .= print_user_picture($user,$vpl->get_course()->id,TRUE,FALSE,TRUE).' '.fullname($user);
		}
		return $ret;
	}
	public function can_access(){
		return true;
	}
	public function link_parms($t){
		return array('type'.$t=>1,'subid'.$t=>$this->subid,'filename'.$t => $this->filename);
	}
}


/**
 * Similarity preprocesing information of a file
 * from any source (directory, zip file or vpl activity)
 *
 */
class vpl_similarity_base{
	var $from;
	var $size;
	static $value_converter=array(); //array to convert string operators to numbers
	var $vecfrec;
	var $hashses;
	protected static function get_value_id($value){
		if(!isset(self::$value_converter[$value])){
			self::$value_converter[$value]=count(self::$value_converter);
		}
		return self::$value_converter[$value];
	}
	public function get_type(){
		return 0;
	}
	public function get_tokenizer(){
		return vpl_tokenizer_factory::get('base');
	}
	public function sintax_normalize(&$tokens){
		return $tokens;
	}
	public function init(&$data,&$from){
		$HASH_SIZE = 4;
		$HASH_REDUCTION = 1000;
		$this->from=$from;
		$this->size=0;
		$this->vecfrec = array();
		$this->hashes = array();
		//Get tokenizer for this language
		$tok = $this->get_tokenizer();
		//Parse code
		$tok->parse($data);
		//Normalize parsed data transforming or removing code
		$tokens = $this->sintax_normalize($tok->get_tokens());
		//Process tokens to get vector of frecuencies and size
		//Calculate hashes
		//TODO improve performance using array of ints and numering operators
		$last = array();
		for($i=0; $i<$HASH_SIZE ; $i++){
			$last[$i] = '';
		}
		foreach($tokens as $token){
			if($token->type == vpl_token_type::operator){
				//Calculate hashes table
				for($i=0; $i<$HASH_SIZE-1 ; $i++){
					$last[$i] = $last[$i+1];
				}
				$last[$HASH_SIZE-1] = $token->value;
				$item = '';
				for($i=0; $i<$HASH_SIZE ; $i++){
					$item .= $last[$i];
				}
				$hash = crc32($item);
				if(isset($this->hashes[$hash%$HASH_REDUCTION])){
					$this->hashes[$hash%$HASH_REDUCTION]++;
				}else{
					$this->hashes[$hash%$HASH_REDUCTION] = 1;
				}
				$this->size++;
				//Get operator id
				$vid = self::get_value_id($token->value);
				if(isset($this->vecfrec[$vid])){
					$this->vecfrec[$vid]++;
				}else{
					$this->vecfrec[$vid]=1;
				}
			}
		}
	}
	public function show_info(){
		return $this->from->show_info();
	}
	public function can_access(){
		return $this->from->can_access();
	}
	public function get_userid(){
		return $this->from->get_userid();
	}
	public function link_parms($t){
		return $this->from->link_parms($t);
	}


	/**
	 * Get similarity 1 among this file and other
	 * @param $other the other file info object
	 * @return number 0-100 %
	 */
	public function similarity1(&$other){
		$dif1=0;
		$taken=0;
		foreach($this->vecfrec as $op => $frec){
			if(isset($other->vecfrec[$op])){
				if($frec != $other->vecfrec[$op]){
					$dif1++;
				}
				$taken++;
			}else{
				$dif1++;
			}
		}
		$dif2 = count($other->vecfrec)-$taken;
		return 100*(1-(($dif1+$dif2)/(count($this->vecfrec)+count($other->vecfrec))));
	}

	/**
	 * Get similarity 2 among this file and other
	 * @param $other the other file info object
	 * @return number 0-100 %
	 */
	public function similarity2(&$other){
		$dif=0;
		$taken=0;
		foreach($this->vecfrec as $op => $frec){
			if(isset($other->vecfrec[$op])){
				$dif += abs($other->vecfrec[$op]-$frec);
				$taken +=$other->vecfrec[$op];
			}else{
				$dif+=$frec;
			}
		}
		$dif += $other->size - $taken;
		return 100*(1-($dif/($this->size+$other->size)));
	}

	/**
	 * Get similarity 3 among this file and other
	 * @param $other the other file info object
	 * @return number 0-100 %
	 */
	public function similarity3(&$other){
		$dif=0;
		$taken=0;
		foreach($this->hashes as $hash => $frec){
			if(isset($other->hashes[$hash])){
				$dif += abs($other->hashes[$hash]-$frec);
				$taken +=$other->hashes[$hash];
			}else{
				$dif+=$frec;
			}
		}
		$dif += $other->size - $taken;
		return 100*(1-($dif/($this->size+$other->size)));
	}
}

//TODO refactor to a protected class
class vpl_files_pair{
	static $id_counter=0;
	static $min_s1=100;
	static $min_s2=100;
	static $min_s3=100;
	public $first;
	public $second;
	public $selected;
	public $s1;
	public $s2;
	public $s3;
	public $id;
	private $cluster_number;
	public function __construct($first=null,$second=null,$s1=0,$s2=0,$s3=0){
		$this->first = $first;
		$this->second = $second;
		$this->selected = false;
		$this->s1 = $s1;
		$this->s2 = $s2;
		$this->s3 = $s3;
		$this->id = self::$id_counter++;
		$this->cluster_number=0;
	}

	static public function set_mins($s1,$s2,$s3){
		self::$min_s1 = $s1;
		self::$min_s2 = $s2;
		self::$min_s3 = $s3;
	}

	static public function cmp($a,$b){
		$al = $a->get_level();
		$bl = $b->get_level();
		if($al == $bl){
			return 0;
		}
		return $al > $bl ? 1 : -1;
	}

	public function get_link(){
		$link = (int)$this->s1.'/'.(int)$this->s2.'/ '.(int)$this->s3;
		$link .= '/ ';
		$link .= $this->s1>=self::$min_s1?'*':'';
		$link .= $this->s2>=self::$min_s2?'*':'';
		$link .= $this->s3>=self::$min_s3?'*':'';
		if($this->first->can_access() && $this->second->can_access()){
			$url = vpl_mod_href('similarity/diff.php','id',required_param('id', PARAM_INT));
			foreach($this->first->link_parms('1') as $parm => $value){
				$url = vpl_url_add_param($url,$parm,$value);
			}
			foreach($this->second->link_parms('2') as $parm => $value){
				$url = vpl_url_add_param($url,$parm,$value);
			}
			$link = link_to_popup_window($url,'viewsub'.$this->id,$link,800,900,null,null,true);
		}
		$HTML = '<div class="vpl_sim'.(int)$this->get_level().'">';
		$HTML .= $link;
		$HTML .= '</div>';
		return $HTML;
	}

	public function get_level(){
		if(!isset($this->level)){
			$level1 = ((100-$this->s1)/(101-self::$min_s1))*11;
			$level2 = ((100-$this->s2)/(101-self::$min_s2))*11;
			$level3 = ((100-$this->s3)/(101-self::$min_s3))*11;
			$this->level = min($level1,$level2,$level3,11);
		}
		return $this->level;
	}

	public function set_cluster($value){
		$this->cluster_number=$value;
	}

	public function get_cluster(){
		if($this->cluster_number>0){
			return '<a href="#clu'.$this->cluster_number.'">'.$this->cluster_number.'</a>';
		}else{
			return '';
		}
	}
}

/**
 * Utility class to get list of preprocessed files
 *
 */
class vpl_similarity{
	static public function scan_activity(&$simil, $vpl, $filesselected=array(), $SPB){
		global $CFG;
		$vpl->require_capability(VPL_SIMILARITY_CAPABILITY);
		$list = get_users_by_capability($vpl->get_context(), 'moodle/legacy:student', '', "u.lastname ASC", '', '', '', '', false);
		if($list == null){
			return;
		}
		$submissions = $vpl->all_last_user_submission();
		$SPB->set_max(count($list));
		$i = 0;
		foreach ($list as $userinfo) {
			$i++;
			$SPB->set_value($i);
			if(isset($submissions[$userinfo->id])){
				$origin = '';
				$subinstance = $submissions[$userinfo->id];
				$submission = new mod_vpl_submission($vpl,$subinstance);
				$subf = $submission->get_submitted_fgm();
				$filelist = $subf->getFileList();
				foreach($filelist as $filename){
					if(count($filesselected)>0 && !isset($filesselected[$filename])){
						continue;
					}
					$sim = vpl_similarity_factory::get($filename);
					if($sim){
						$data = $subf->getFileData($filename);
						$from =new vpl_file_from_activity($filename,$vpl,$subinstance);
						$sim->init($data,$from);
						if($sim->size>10){
							$simil[]=$sim;
						}
					}
				}
			}
		}
	}

	static public function scan_zip(&$simil, $zipfile, $vpl, $filesselected=array(), $SPB){
		global $CFG;
		$ext = strtoupper(pathinfo($zipfile,PATHINFO_EXTENSION));
		if($ext != 'ZIP'){
			print_error('nozipfile');
		}
		//TODO change to new zip class when available
		//Get class to handle zip
		require_once($CFG->libdir . '/filelib.php');
		require_once($CFG->libdir . '/pclzip/pclzip.lib.php');
		$course = $vpl->get_course();
		$dirfilename=tempnam($CFG->dataroot . '/temp/'  , 'vpl_dirextract' );
		$destdir=$dirfilename.'.dir';
		mkdir($destdir,0700);
		$zipfilename=$CFG->dataroot.'/'.$course->id.'/'.$zipfile;
		debugging("Unzip file ".$zipfilename, DEBUG_DEVELOPER);
		$zip = new PclZip($zipfilename);
		$SPB->set_value(get_string('unzipping',VPL));
		$listfiles = $zip->extract(PCLZIP_OPT_PATH, $destdir);
		$toremove=strlen($destdir);
		$SPB->set_max(count($listfiles));
		$i=1;
		foreach($listfiles as $file){
			$SPB->set_value($i++);
			if(!$file['folder']){
				$filename=$file['filename'];
				debugging("Examining file ".$filename, DEBUG_DEVELOPER);
				//TODO remove if no GAP file
				vpl_file_from_zipfile::process_gap_userfile($filename);
				if(count($filesselected)>0 && !isset($filesselected[basename($filename)])){
					continue;
				}
				$sim = vpl_similarity_factory::get($filename);
				if($sim){
					$data = file_get_contents($filename);
					//TODO locate userid
					$from =new vpl_file_from_zipfile(substr($filename,$toremove),$zipfile);
					$sim->init($data,$from);
					if($sim->size>10){
						$simil[]=$sim;
					}
				}
			}
		}
		//remove temp dir
		$SPB->set_value(get_string('cleaningtempdata').' '.count($simil));
		unlink($dirfilename);
		vpl_delete_dir($destdir);
		$SPB->set_value(count($listfiles));
	}
	static function get_dir_filelist($basepath,$dir){
		$filelist = array();
		$dirlist = array();
		if($dir > ''){
			$basedir = $basepath.'/'.$dir;
		}else{
			$basedir = $basepath;
		}
		if($dd = opendir($basedir)){
			while ($file=readdir($dd)) {
				if ($file[0] == '.'){
					continue;
				}
				$relpath =$dir.'/'.$file;
				$fullpath=$basedir.'/'.$file;
				if(is_file($fullpath)){
					$filelist[] = $relpath;
				}elseif(is_dir($fullpath)){
					$dirlist[] = $relpath;
				}
			}
			closedir($dd);
		}
		foreach($dirlist as $adir){
			$otherlist = self::get_dir_filelist($basepath,$adir);
			$filelist = array_merge($filelist,$otherlist);
		}
		return $filelist;
	}
	static public function scan_directory(&$simil, $dir, $vpl, $filesselected=array(), $SPB){
		global $CFG;
		$basedir = $CFG->dataroot.'/'.$vpl->get_course()->id;
		$SPB->set_value(get_string('scanningdir',VPL));
		$filelist = self::get_dir_filelist($basedir.'/'.$dir,'');
		$SPB->set_value((string)count($filelist));
		$SPB->set_max(count($filelist));
		foreach($filelist as $pos => $file){
			$SPB->set_value($pos+1);
			$base = basename($file);
			//TODO remove if no GAP file
			$fullpath = $basedir.'/'.$dir.'/'.$file;
			vpl_file_from_dir::process_gap_userfile($fullpath);
			if(count($filesselected)>0 && !isset($filesselected[$base])){
				continue;
			}
			$sim = vpl_similarity_factory::get($file);
			if($sim){
				$data = file_get_contents($fullpath);
				//TODO locate userid
				$from =new vpl_file_from_dir($file,$dir);
				$sim->init($data,$from);
				if($sim->size>10){
					$simil[]=$sim;
				}
			}
		}
	}

	static public function get_selected(&$files, $maxselected, $slimit, $SPB){
		$vs1=array();
		$vs2=array();
		$vs3=array();
		$minlevel1=0;
		$minlevel2=0;
		$minlevel3=0;
		$selected = array();
		$jlimit=count($files);
		$SPB->set_max($slimit);
		for($i=0; $i<$slimit; $i++){ //Search similarity with
			$SPB->set_value($i+1);
			$current=$files[$i];
			$current_type=$current->get_type();
			$userid = $current->get_userid();
			for($j=$i+1;$j<$jlimit; $j++){ //Compare with all others
				$other=$files[$j];
				//If not the same language then skip
				if($current_type != $other->get_type() ||
				(strlen($userid)>0 && $userid == $other->get_userid())){
					continue;
				}
				//Calculate metrics
				$s1=$current->similarity1($other);
				$s2=$current->similarity2($other);
				$s3=$current->similarity3($other);
				if($s1>=$minlevel1 || $s2>=$minlevel2 || $s3 >= $minlevel3){
					$case = new vpl_files_pair($files[$i],$files[$j],$s1,$s2,$s3);
					if($s1>=$minlevel1){
						$vs1[] = $case;
						if(count($vs1) > 2*$maxselected){
							self::filter_selected($vs1,$maxselected,$minlevel1,1);
						}
					}
					if($s2>=$minlevel2){
						$vs2[] = $case;
						if(count($vs2) > 2*$maxselected){
							self::filter_selected($vs2,$maxselected,$minlevel2,2);
						}
					}
					if($s3>=$minlevel3){
						$vs3[] = $case;
						if(count($vs3) > 2*$maxselected){
							self::filter_selected($vs3,$maxselected,$minlevel3,3);
						}
					}
				}
			}
		}
		self::filter_selected($vs1,$maxselected,$minlevel1,1,true);
		self::filter_selected($vs2,$maxselected,$minlevel2,2,true);
		self::filter_selected($vs3,$maxselected,$minlevel3,3,true);
		vpl_files_pair::set_mins($minlevel1,$minlevel2,$minlevel3);
		//Merge vs1, vs2 and vs3
		$max = count($vs1);
		for($i=0; $i< $max ; $i++){
			if(! $vs1[$i]->selected){
				$selected[]=$vs1[$i];
				$vs1[$i]->selected=true;
			}
			if(! $vs2[$i]->selected){
				$selected[]=$vs2[$i];
				$vs2[$i]->selected=true;
			}
			if(! $vs3[$i]->selected){
				$selected[]=$vs3[$i];
				$vs3[$i]->selected=true;
			}
		}
		//usort of old PHP versions don't call static class functions
		$corder = new vpl_files_pair;
		usort($selected,array($corder,'cmp'));
		return $selected;
	}
	static $corder=null;
	static public function filter_selected(&$vec, $maxselected, &$minlevel,$sid, $last=false){
		if(count($vec)>$maxselected || $last){
			if(self::$corder === null){
				self::$corder = new vpl_similarity;
			}
			//usort of old PHP versions don't call static class functions
			if(!usort($vec,array(self::$corder,'cmp_selected'.$sid))){
				debugging('usort error');
			}
			$field = 's'.$sid;
			$vec = array_slice($vec,0,$maxselected);
			$minlevel=$vec[count($vec)-1]->$field;
		}
	}
	static public function cmp_selected1($a, $b){
		if ($a->s1 == $b->s1) {
			if ($a->s3 == $b->s3) {
				if ($a->s2 == $b->s2) {
					return 0;
				}
				return ($a->s2 > $b->s2) ? -1 : 1;
			}
			return ($a->s3 > $b->s3) ? -1 : 1;
		}
		return ($a->s1 > $b->s1) ? -1 : 1;
	}

	static public function cmp_selected2($a, $b){
		if ($a->s2 == $b->s2) {
			if ($a->s1 == $b->s1) {
				if ($a->s3 == $b->s3) {
					return 0;
				}
				return ($a->s3 > $b->s3) ? -1 : 1;
			}
			return ($a->s1 > $b->s1) ? -1 : 1;
		}
		return ($a->s2 > $b->s2) ? -1 : 1;
	}

	static public function cmp_selected3($a, $b){
		if ($a->s3 == $b->s3) {
			if ($a->s1 == $b->s1) {
				if ($a->s2 == $b->s2) {
					return 0;
				}
				return ($a->s2 > $b->s2) ? -1 : 1;
			}
			return ($a->s1 > $b->s1) ? -1 : 1;
		}
		return ($a->s3 > $b->s3) ? -1 : 1;
	}
}
?>