<?php
/**
 * @version		$Id: similarity_test.php,v 1.3 2009/11/20 19:18:35 juanca Exp $
 * @package		vpl. similarity test
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino
 **/

/**
 * List all instances in a course
 *
 * @author  Juan Carlos Rodriguez del Pino
 * @version $Id: similarity_test.php,v 1.3 2009/11/20 19:18:35 juanca Exp $
 * @package vpl
 **/

require_once dirname(__FILE__).'/../../../config.php';
require_once dirname(__FILE__).'/../lib.php';
require_once dirname(__FILE__).'/../vpl.class.php';
require_once dirname(__FILE__).'/../vpl_submission.class.php';
require_once dirname(__FILE__).'/watermark.php';
require_once $CFG->libdir.'/formslib.php';
require_once dirname(__FILE__).'/similarity_form.class.php';

$id = required_param('id', PARAM_INT);
$vpl = new mod_vpl($id);
$course = $vpl->get_course();
$vpl->require_capability(VPL_SIMILARITY_CAPABILITY);
$vpl->add_to_log('similarity form', vpl_mod_href('similarity/similarity_form.php','id',$id));
$form = new vpl_similarity_form('listsimilarity.php',$vpl);
//Print header
$vpl->print_header(get_string('similarity',VPL));
$vpl->print_view_tabs(basename(__FILE__));
if($fromform = $form->get_data()){
		$code1='
	#include "conjuntoenteros.h"
#include<iostream>
#include<cmath>
using namespace std;

//Constructor por omision, permite crear conjuntos vacios
ConjuntoDeEnteros::ConjuntoDeEnteros()
{
	primero=NULL;
	tamConj=0;
}
//constructor que admite como parámetro un entero y que inicializa el conjunto unitario con el valor pasado.
ConjuntoDeEnteros::ConjuntoDeEnteros(int elem)
{
	tamConj=0;
	primero=NULL;
	anade(elem);
}
//Constructor que permite inicializar un objeto con un vector de enteros de un tamaño determinado, pasado por parametro. El orden de los parametros es, primero el vector y luego el tamaño del vector, el conjunto no permite elementos repetidos
ConjuntoDeEnteros::ConjuntoDeEnteros(const int vecele[],int tam)
{
	int i;
	tamConj=0;
	primero=NULL;
	for(i=0;i<tam;i++)
	{
		anade(vecele[i]);
	}
}
//Destructor del conjunto, libera la memoria ocupada por el objeto
ConjuntoDeEnteros::~ConjuntoDeEnteros()
{
	struct nod *aux;
	while(primero!=NULL)
	{
		aux=primero;
		primero=primero->sig;
		delete aux;
	}
	tamConj=0;
}
/*Función "cardinal" que devuelve el número de elementos del conjunto.*/
int ConjuntoDeEnteros::cardinal()const
{
	return tamConj;
}
/*Función "estaVacio" que devuelve verdadero si el conjunto está vacío y falso en caso contrario.*/
bool ConjuntoDeEnteros::estaVacio()const
{
	if (cardinal() != 0) return false;
	else return true;
}
/*Función "anade" que añade al conjunto un nuevo elemento, devolviendo verdadero si lo añadió
y falso en caso de que no lo haya hecho, porque ya existe ese elemento en el conjunto*/
//El nuevo elemento se añande al principio de la lista
bool ConjuntoDeEnteros::anade(int elem)
{
	if(pertenece(elem))
	return false;
	struct nod *aux;
	aux=new struct nod;
	aux->valor=elem;
	aux->sig=primero;
	primero=aux;
	tamConj=tamConj+1;
	return true;
}
//Una función "elimina" que elimine del conjunto un elemento pasado por parámetro, devolviendo verdadero si lo eliminó y falso en caso de que no esté en el conjunto.
bool ConjuntoDeEnteros::elimina(int elem)
{
	struct nod *aux1, *aux2;
	bool iguales=false;
	if (pertenece(elem))//comprobar si el elemento pertenece al conjunto
	{
		//comprobar si el elemento es el primero del conjunto
		aux1=primero;
		if (aux1->valor==elem)
		{
			aux2=aux1;
			primero=primero->sig;
		}
		else
		{
			//busqueda de la posicion del elemento en el conjunto
			while(!iguales)
			{
				if(aux1->sig->valor==elem)
				{
					aux2=aux1->sig;
					aux1->sig=aux2->sig;
					iguales=true;
				}
				else
					aux1=aux1->sig;
			}
		}
		delete aux2;
		tamConj=tamConj-1;
		return true;
	}
	else
	return false;
}
/*Función "pertenece" que devuelve verdadero si el entero pasado por parámetro pertenece al
conjunto y falso en caso contrario.*/
bool ConjuntoDeEnteros::pertenece(int elem)const
{
	if(primero==NULL) return false;
	struct nod *aux;
	aux=primero;
	while((aux!=NULL) && (aux->valor !=elem))
	{
		aux=aux->sig;
	}
	if(aux==NULL)return false;
	return true;
}
/*Procedimiento "elementos" al que se le pasa un vector de enteros y lo modifica rellenándolo
desde el principio con los elementos del conjunto. El vector pasado tiene que ser lo suficientemente
grande para almacenar los elementos del conjunto.*/
void ConjuntoDeEnteros::elementos(int vecent[])const
{
	int i;
	struct nod *aux;
	aux=primero;
	for(i=0;i<cardinal();i++)
	{
		vecent[i]=aux->valor;
		aux=aux->sig;
	}
}
//sobrecarga del operador de asignacion
ConjuntoDeEnteros & ConjuntoDeEnteros::operator =(const ConjuntoDeEnteros &conjA)
{
	struct nod *aux1, *aux2;
	while(primero!=NULL)
	{
		aux1=primero;
		primero=primero->sig;
		delete aux1;
	}
	tamConj=0;
	aux2=conjA.primero;
	while(aux2!=NULL)
	{
		anade(aux2->valor);
		aux2=aux2->sig;
	}
	return *this;
}
//constructor de copia
ConjuntoDeEnteros::ConjuntoDeEnteros(const ConjuntoDeEnteros &conjA)
{
	struct nod *aux;
	tamConj=0;
	primero=NULL;
	aux=conjA.primero;
	while(aux!=NULL)
	{
		anade(aux->valor);
		aux=aux->sig;
	}
}
/*Función no miembro que sobrecarga el operador + para realizar la operación de unión
de conjuntos. A+B devuelve un conjunto donde se encuentran todos los elementos de A y de B*/
ConjuntoDeEnteros operator+(const ConjuntoDeEnteros conjA, const ConjuntoDeEnteros conjB)
{
	ConjuntoDeEnteros conjUnion;
	if ((conjA.primero==NULL) && (conjB.primero==NULL))
		return conjUnion;
	struct nod *aux1, *aux2;
	aux1=conjA.primero;
	aux2=conjB.primero;	
	while(aux1!=NULL)
	{
		conjUnion.anade(aux1->valor);
		aux1=aux1->sig;
	}
	while(aux2!=NULL)
	{
		conjUnion.anade(aux2->valor);
		aux2=aux2->sig;
	}
	return conjUnion;
}
/*Una función no miembro que sobrecargue el operador == para realizar la comparación de igualdad entre conjuntos. A==B devuelve verdadero si todos los elementos de A están en B y todos los de B están en A. O sea, que A y B tienen los mismos elementos.*/
bool operator==(const ConjuntoDeEnteros conjA, const ConjuntoDeEnteros conjB)
{
	struct nod *aux1;
	aux1=conjA.primero;
	if (conjA.cardinal()!= conjB.cardinal()) return false;
	while (aux1!=NULL)
	{
		if (not conjB.pertenece(aux1->valor))return false;
		aux1=aux1->sig;
	}
	return true;
}
/*Una función no miembro que sobrecargue el operador < para realizar la operación de "contenido" entre conjuntos. A<B devuelve verdadero si todos los elementos de A están en B.*/
bool operator<(const ConjuntoDeEnteros conjA, const ConjuntoDeEnteros conjB)
{
	if ((((conjA.cardinal()==0)&& (conjB.cardinal()==0))||(conjA.cardinal()==0))) return true;
	if(conjB.cardinal()<conjA.cardinal()) return false;
	struct nod *aux;
	aux=conjA.primero;
	while(aux!=NULL)
	{
		if (not conjB.pertenece(aux->valor))return false;
		aux=aux->sig;
	}
	return true;
}
/*Una función no miembro que sobrecargue el operador * para realizar la operación de intersección de conjuntos. A*B devuelve el conjunto intersección, o sea, un conjunto formado por los elementos que pertenecen a ambos conjuntos.*/
ConjuntoDeEnteros operator*(const ConjuntoDeEnteros conjA, const ConjuntoDeEnteros conjB)
{
	ConjuntoDeEnteros conjInter;
	if ((conjA.estaVacio()) && (conjB.estaVacio())) return conjInter;
	struct nod *aux;
	aux=conjB.primero;
	while(aux!=NULL)
	{
		if (conjA.pertenece(aux->valor))
		{
			conjInter.anade(aux->valor);
		}
		aux=aux->sig;
	}
	return conjInter;
}
/*Una función no miembro que sobrecargue el operador - para realizar la diferencia de conjuntos. A-B devuelve un conjunto que contenga los elementos de A que no pertenecen a B.*/
ConjuntoDeEnteros operator-(const ConjuntoDeEnteros conjA, const ConjuntoDeEnteros conjB)
{
	ConjuntoDeEnteros conjDif;
	if(conjA.estaVacio()) return conjDif;
	struct nod *aux;
	aux=conjA.primero;
	while (aux!=NULL)
	{
		if(not conjB.pertenece(aux->valor))
		{
			conjDif.anade(aux->valor);			
		}
		aux=aux->sig;
	}
	return conjDif;
}
';
		
	$code2='
	#include "conjuntoenteros.h"
#include<iostream>
#include<cmath>
using namespace std;

//Constructor por omision, permite crear conjuntos vacios
ConjuntoDeEnteros::ConjuntoDeEnteros()
{
	primero=NULL;
	tamConj=0;
}
//constructor que admite como parámetro un entero y que inicializa el conjunto unitario con el valor pasado.
ConjuntoDeEnteros::ConjuntoDeEnteros(int elem)
{
	tamConj=0;
	primero=NULL;
	anade(elem);
}
//Constructor que permite inicializar un objeto con un vector de enteros de un tamaño determinado, pasado por parametro. El orden de los parametros es, primero el vector y luego el tamaño del vector, el conjunto no permite elementos repetidos
ConjuntoDeEnteros::ConjuntoDeEnteros(const int vecele[],int tam)
{
	int i;
	tamConj=0;
	primero=NULL;
	for(i=0;i<tam;i++)
	{
		anade(vecele[i]);
	}
}
//Destructor del conjunto, libera la memoria ocupada por el objeto
ConjuntoDeEnteros::~ConjuntoDeEnteros()
{
	struct nod *aux;
	while(primero!=NULL)
	{
		aux=primero;
		primero=primero->sig;
		delete aux;
	}
	tamConj=0;
}
/*Función "cardinal" que devuelve el número de elementos del conjunto.*/
int ConjuntoDeEnteros::cardinal()const
{
	return tamConj;
}
/*Función "estaVacio" que devuelve verdadero si el conjunto está vacío y falso en caso contrario.*/
bool ConjuntoDeEnteros::estaVacio()const
{
	if (cardinal() != 0) return false;
	else return true;
}
/*Función "anade" que añade al conjunto un nuevo elemento, devolviendo verdadero si lo añadió
y falso en caso de que no lo haya hecho, porque ya existe ese elemento en el conjunto*/
//El nuevo elemento se añande al principio de la lista
bool ConjuntoDeEnteros::anade(int elem)
{
	if(pertenece(elem))
	return false;
	struct nod *aux;
	aux=new struct nod;
	aux->valor=elem;
	aux->sig=primero;
	primero=aux;
	tamConj=tamConj+1;
	return true;
}
//Una función "elimina" que elimine del conjunto un elemento pasado por parámetro, devolviendo verdadero si lo eliminó y falso en caso de que no esté en el conjunto.
bool ConjuntoDeEnteros::elimina(int elem)
{
	struct nod *aux1, *aux2;
	bool iguales=false;
	if (pertenece(elem))//comprobar si el elemento pertenece al conjunto
	{
		//comprobar si el elemento es el primero del conjunto
		aux1=primero;
		if (aux1->valor==elem)
		{
			aux2=aux1;
			primero=primero->sig;
		}
		else
		{
			//busqueda de la posicion del elemento en el conjunto
			while(!iguales)
			{
				if(aux1->sig->valor==elem)
				{
					aux2=aux1->sig;
					aux1->sig=aux2->sig;
					iguales=true;
				}
				else
					aux1=aux1->sig;
			}
		}
		delete aux2;
		tamConj=tamConj-1;
		return true;
	}
	else
	return false;
}
/*Función "pertenece" que devuelve verdadero si el entero pasado por parámetro pertenece al
conjunto y falso en caso contrario.*/
bool ConjuntoDeEnteros::pertenece(int elem)const
{
	if(primero==NULL) return false;
	struct nod *aux;
	aux=primero;
	while((aux!=NULL) && (aux->valor !=elem))
	{
		aux=aux->sig;
	}
	if(aux==NULL)return false;
	return true;
}
/*Procedimiento "elementos" al que se le pasa un vector de enteros y lo modifica rellenándolo
desde el principio con los elementos del conjunto. El vector pasado tiene que ser lo suficientemente
grande para almacenar los elementos del conjunto.*/
void ConjuntoDeEnteros::elementos(int vecent[])const
{
	int i;
	struct nod *aux;
	aux=primero;
	for(i=0;i<cardinal();i++)
	{
		vecent[i]=aux->valor;
		aux=aux->sig;
	}
}
//sobrecarga del operador de asignacion
ConjuntoDeEnteros & ConjuntoDeEnteros::operator =(const ConjuntoDeEnteros &conjA)
{
	struct nod *aux1, *aux2;
	while(primero!=NULL)
	{
		aux1=primero;
		primero=primero->sig;
		delete aux1;
	}
	tamConj=0;
	aux2=conjA.primero;
	while(aux2!=NULL)
	{
		anade(aux2->valor);
		aux2=aux2->sig;
	}
	return *this;
}
//constructor de copia
ConjuntoDeEnteros::ConjuntoDeEnteros(const ConjuntoDeEnteros &conjA)
{
	struct nod *aux;
	tamConj=0;
	primero=NULL;
	aux=conjA.primero;
	while(aux!=NULL)
	{
		anade(aux->valor);
		aux=aux->sig;
	}
}
/*Función no miembro que sobrecarga el operador + para realizar la operación de unión
de conjuntos. A+B devuelve un conjunto donde se encuentran todos los elementos de A y de B*/
ConjuntoDeEnteros operator+(const ConjuntoDeEnteros conjA, const ConjuntoDeEnteros conjB)
{
	ConjuntoDeEnteros conjUnion;
	if ((conjA.primero==NULL) && (conjB.primero==NULL))
		return conjUnion;
	struct nod *aux1, *aux2;
	aux1=conjA.primero;
	aux2=conjB.primero;	
	while(aux1!=NULL)
	{
		conjUnion.anade(aux1->valor);
		aux1=aux1->sig;
	}
	while(aux2!=NULL)
	{
		conjUnion.anade(aux2->valor);
		aux2=aux2->sig;
	}
	return conjUnion;
}
/*Una función no miembro que sobrecargue el operador == para realizar la comparación de igualdad entre conjuntos. A==B devuelve verdadero si todos los elementos de A están en B y todos los de B están en A. O sea, que A y B tienen los mismos elementos.*/
bool operator==(const ConjuntoDeEnteros conjA, const ConjuntoDeEnteros conjB)
{
	struct nod *aux1;
	aux1=conjA.primero;
	if (conjA.cardinal()!= conjB.cardinal()) return false;
	while (aux1!=NULL)
	{
		if (not conjB.pertenece(aux1->valor))return false;
		aux1=aux1->sig;
	}
	return true;
}
/*Una función no miembro que sobrecargue el operador < para realizar la operación de "contenido" entre conjuntos. A<B devuelve verdadero si todos los elementos de A están en B.*/
bool operator<(const ConjuntoDeEnteros conjA, const ConjuntoDeEnteros conjB)
{
	if ((((conjA.cardinal()==0)&& (conjB.cardinal()==0))||(conjA.cardinal()==0))) return true;
	if(conjB.cardinal()<conjA.cardinal()) return false;
	struct nod *aux;
	aux=conjA.primero;
	while(aux!=NULL)
	{
		if (not conjB.pertenece(aux->valor))return false;
		aux=aux->sig;
	}
	return true;
}
/*Una función no miembro que sobrecargue el operador * para realizar la operación de intersección de conjuntos. A*B devuelve el conjunto intersección, o sea, un conjunto formado por los elementos que pertenecen a ambos conjuntos.*/
ConjuntoDeEnteros operator*(const ConjuntoDeEnteros conjA, const ConjuntoDeEnteros conjB)
{
	ConjuntoDeEnteros conjInter;
	if ((conjA.estaVacio()) && (conjB.estaVacio())) return conjInter;
	struct nod *aux;
	aux=conjB.primero;
	while(aux!=NULL)
	{
		if (conjA.pertenece(aux->valor))
		{
			conjInter.anade(aux->valor);
		}
		aux=aux->sig;
	}
	return conjInter;
}
/*Una función no miembro que sobrecargue el operador - para realizar la diferencia de conjuntos.
 A-B devuelve un conjunto que contenga los
  elementos de A que no pertenecen a B.
ConjuntoDeEnteros operator-(const ConjuntoDeEnteros conjA, const ConjuntoDeEnteros conjB)
{
	ConjuntoDeEnteros conjDif;
	if(conjA.estaVacio()) return conjDif;
	struct nod *aux;
	aux=conjA.primero;
	while (aux!=NULL)
	{
		if(not conjB.pertenece(aux->valor))
		{
			conjDif.anade(aux->valor);			
		}
		aux=aux->sig;
	}
	return conjDif;
}*/
';
	require_once('tokenizer_cpp.class.php');
	require_once('similarity_base.class.php');
	$tokenizer = new vpl_tokenizer_cpp();
	$tokenizer->parse($code1);
	$simil1 = new vpl_similarity_base();
	$simil1->init($tokenizer->get_tokens());
	$simil2 = new vpl_similarity_base();
	$tokenizer->parse($code2);
	print_box_start();
	$tokenizer->show_tokens();
	print_box_end();
	$simil2->init($tokenizer->get_tokens());
	print_box_start();
	$tokenizer->show_tokens();
	echo $simil1->similarity1($simil1).'<br />';
	echo $simil1->similarity1($simil2).'<br />';
	echo $simil1->similarity2($simil1).'<br />';
	echo $simil1->similarity2($simil2).'<br />';
	print_box_end();
}else{
	$form->display();
}
$vpl->print_footer();
?>